Package: rstpm2
Type: Package
Title: Smooth Survival Models, Including Generalized Survival Models
Authors@R: c(person("Mark", "Clements", role = c("aut", "cre"),
		     email = "mark.clements@ki.se"),
	      person("Xing-Rong", "Liu", role = "aut",
		     email = "xingrong.liu@ki.se"),
	      person("Benjamin", "Christoffersen", role = "aut",
		     email = "benjamin.christoffersen@ki.se"),
	      person("Paul", "Lambert", role = "ctb", email="pl4@leicester.ac.uk"),
	      person("Lasse", "Hjort Jakobsen", role = "ctb", email="lasse.j@rn.dk"),
	      person("Alessandro", "Gasparini", role = "ctb"),
	      person("Gordon","Smyth", role="cph"),
	      person("Patrick","Alken", role="cph"),
	      person("Simon","Wood", role="cph"),
	      person("Rhys","Ulerich", role="cph"))
Version: 1.7.1
Depends: R (>= 3.0.2), methods, survival, splines
Imports: graphics, Rcpp (>= 0.10.2), stats, mgcv, bbmle (>= 1.0.20),
        fastGHQuad, utils, parallel, mvtnorm, numDeriv, lsoda
Suggests: eha, testthat, ggplot2, lattice, readstata13, mstate, scales,
        survPen, flexsurv, timereg
LinkingTo: Rcpp,RcppArmadillo
Maintainer: Mark Clements <mark.clements@ki.se>
Description: R implementation of generalized survival models (GSMs), smooth accelerated failure time (AFT) models and Markov multi-state models. For the GSMs, g(S(t|x))=eta(t,x) for a link function g, survival S at time t with covariates x and a linear predictor eta(t,x). The main assumption is that the time effect(s) are smooth <doi:10.1177/0962280216664760>. For fully parametric models with natural splines, this re-implements Stata's 'stpm2' function, which are flexible parametric survival models developed by Royston and colleagues. We have extended the parametric models to include any smooth parametric smoothers for time. We have also extended the model to include any smooth penalized smoothers from the 'mgcv' package, using penalized likelihood. These models include left truncation, right censoring, interval censoring, gamma frailties and normal random effects <doi:10.1002/sim.7451>, and copulas. For the smooth AFTs, S(t|x) = S_0(t*eta(t,x)), where the baseline survival function S_0(t)=exp(-exp(eta_0(t))) is modelled for natural splines for eta_0, and the time-dependent cumulative acceleration factor eta(t,x)=\int_0^t exp(eta_1(u,x)) du for log acceleration factor eta_1(u,x). The Markov multi-state models allow for a range of models with smooth transitions to predict transition probabilities, length of stay, utilities and costs, with differences, ratios and standardisation.
URL: https://github.com/mclements/rstpm2
BugReports: https://github.com/mclements/rstpm2/issues
License: GPL-2 | GPL-3
LazyData: yes
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2025-11-12 15:48:59 UTC; marcle
Author: Mark Clements [aut, cre],
  Xing-Rong Liu [aut],
  Benjamin Christoffersen [aut],
  Paul Lambert [ctb],
  Lasse Hjort Jakobsen [ctb],
  Alessandro Gasparini [ctb],
  Gordon Smyth [cph],
  Patrick Alken [cph],
  Simon Wood [cph],
  Rhys Ulerich [cph]
Repository: CRAN
Date/Publication: 2025-11-13 06:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 02:32:13 UTC; unix
Archs: rstpm2.so.dSYM
