inlineCxxPlugin <- 
    Rcpp:::Rcpp.plugin.maker(
               include.before = "#include <splines.h>", 
               libs           = # sprintf(
                   "$(LAPACK_LIBS) $(BLAS_LIBS) $(FLIBS)", ##
                                        ## list.files(system.file("libs", package = "rstpm2"),
                                        ##            pattern = "\\.(so|dll|dylib)$",
                                        ##            full.names = TRUE)),
               package        = "rstpm2")

