# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

vdqagsRcpp <- function(f, a, b, epsrel, epsabs, limit, ny) {
    .Call(`_rstpm2_vdqagsRcpp`, f, a, b, epsrel, epsabs, limit, ny)
}

vdqagiRcpp <- function(f, bound, inf, epsrel, epsabs, limit, ny) {
    .Call(`_rstpm2_vdqagiRcpp`, f, bound, inf, epsrel, epsabs, limit, ny)
}

vrdqk21Rcpp <- function(f, lower, upper, a, b) {
    .Call(`_rstpm2_vrdqk21Rcpp`, f, lower, upper, a, b)
}

vrdqk15Rcpp <- function(f, boun, inf, a, b) {
    .Call(`_rstpm2_vrdqk15Rcpp`, f, boun, inf, a, b)
}

test_vdqags <- function() {
    .Call(`_rstpm2_test_vdqags`)
}

test_vdqagi <- function() {
    .Call(`_rstpm2_test_vdqagi`)
}

vunirootRcpp <- function(f, lower, upper, fa, fb, numiter, tol) {
    .Call(`_rstpm2_vunirootRcpp`, f, lower, upper, fa, fb, numiter, tol)
}

voptimizeRcpp <- function(f, ax, bx, tol) {
    .Call(`_rstpm2_voptimizeRcpp`, f, ax, bx, tol)
}

