\name{legendre.quadrature.rule.200}
\Rdversion{1.1}
\alias{legendre.quadrature.rule.200}
\docType{data}
\title{
Legendre quadrature rule for n=200.
}
\description{
  Legendre quadrature rule for n=200.
}
\usage{data(legendre.quadrature.rule.200)}
\format{
  A data frame with 200 observations on the following 2 variables.
  \describe{
\item{\code{x}}{x values between -1 and 1}
\item{\code{w}}{weights}
  }
}
%% \details{
%% %%  ~~ If necessary, more details than the __description__ above ~~
%% }
%% \source{
%% %%  ~~ reference to a publication or URL from which the data were obtained ~~
%% }
%% \references{
%% %%  ~~ possibly secondary sources and usages ~~
%% }
\examples{
data(legendre.quadrature.rule.200)
## maybe str(legendre.quadrature.rule.200) ; ...
}
\keyword{datasets}
